/******************************************************************************
* C Source: Generic                                                           *
*                                                                             *
* File:                                                                       *
*    generic.c                                                                *
*                                                                             *
* Description:                                                                *
*    This file contains functions for performing various miscellaneous tasks. *
*                                                                             *
* Module List:                                                                *
*    Allocate_Memory                                                          *
*       Allocates memory. If the memory is not available, it handles it as a  *
*       fatal error.                                                          *
*    Convert_String_to_Lower_Case                                             *
*       Convert letters in a string to lower case.                            *
*    Convert_String_to_Upper_Case                                             *
*       Convert letters in a string to upper case.                            *
*    dtotal                                                                   *
*       Sums a double precision array.                                        *
*    Handle_Fatal_Error                                                       *
*       Prints message about what and where a fatal error, and aborts the     *
*       program.                                                              *
*    remquotes                                                                *
*       Strips quotes (') from a string.                                      *
*    strtrim                                                                  *
*       Strips leading blanks from a string.                                  *
*    Translate_Environment_Variable                                           *
*       Returns the translation of an environment variable. If it can't be    *
*       translated, then an empty string is returned.                         *
*     Trim_Trailing_White_Space                                               *
*       Trim trailing white space from a string.                              *
*                                                                             *
* History:                                                                    *
*    Created    : 01-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
*    Updated    : 28-NOV-94 by J. Doggett,                                    *
*       Updated dtotal function.                                              *
*    Redelivered: 30-NOV-94 by J. Doggett,                                    *
*       GetImage Version 1.1                                                  *
*                                                                             *
* Copyright (c) 1993, 1994, Association of Universities for Research in       *
* Astronomy, Inc.  All Rights Reserved.                                       *
* Produced under National Aeronautics and Space Administration grant          *
* NAG W-2166.                                                                 *
******************************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "generic.h"

#include "config.h"

/******************************************************************************
* Module:                                                                     *
*    Allocate_Memory                                                          *
*                                                                             *
* File:                                                                       *
*    generic.c                                                                *
*                                                                             *
* Description:                                                                *
*    This function allocates specified amount of memory using malloc, and     *
*    then handles the error if memory can't be allocated.                     *
*                                                                             *
* Return Value:                                                               *
*    void *: Pointer to allocated memory.                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Size:                                                                 *
*          int, Value                                                         *
*             Amount of memory in bytes to allocate                           *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 01-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/*******************************
* Allocate_Memory declarations *
*******************************/
void *Allocate_Memory( Size )
int   Size;
{
   void* Allocated_Memory; /* Pointer to the allocated memory */

/****************************************
* Allocate_Memory begin executable code *
****************************************/
   Allocated_Memory = (void*)malloc( Size );

   if ( Allocated_Memory == NULL )
      Handle_Fatal_Error( "Allocate_Memory", "Out of Memory!" );

   return Allocated_Memory;
}
/**************************************
* Allocate_Memory end executable code *
**************************************/

/******************************************************************************
* Module:                                                                     *
*    Convert_String_to_Lower_Case                                             *
*                                                                             *
* File:                                                                       *
*    generic.c                                                                *
*                                                                             *
* Description:                                                                *
*    Converts letters in a string to lower case.                              *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments:                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       none                                                                  *
*    Input/Output Arguments:                                                  *
*       String                                                                *
*          STRING, Reference                                                  *
*             String to be converted.                                         *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 25-AUG-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/********************************************
* Convert_String_to_Lower_Case declarations *
********************************************/
void   Convert_String_to_Lower_Case( String )
STRING String;
{
   int Nth_Char; /* Nth character of string we're converting */

/*****************************************************
* Convert_String_to_Lower_Case begin executable code *
*****************************************************/
   for ( Nth_Char = 0; String[ Nth_Char ] != ( char )NULL; Nth_Char++ )
      String[ Nth_Char ] = tolower( String[ Nth_Char ] );

   return;
}
/***************************************************
* Convert_String_to_Lower_Case end executable code *
***************************************************/

/******************************************************************************
* Module:                                                                     *
*    Convert_String_to_Upper_Case                                             *
*                                                                             *
* File:                                                                       *
*    generic.c                                                                *
*                                                                             *
* History:                                                                    *
*    Created    : 25-AUG-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
* Description:                                                                *
*    Converts letters in a string to upper case.                              *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments:                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       none                                                                  *
*    Input/Output Arguments:                                                  *
*       String                                                                *
*          STRING, Reference                                                  *
*             String to be converted.                                         *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 25-AUG-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/********************************************
* Convert_String_to_Upper_Case declarations *
********************************************/
void   Convert_String_to_Upper_Case( String )
STRING String;
{
   int Nth_Char; /* Nth character of string we're converting */

/*****************************************************
* Convert_String_to_Upper_Case begin executable code *
*****************************************************/
   for ( Nth_Char = 0; String[ Nth_Char ] != ( char )NULL; Nth_Char++ )
      String[ Nth_Char ] = toupper( String[ Nth_Char ] );

   return;
}
/***************************************************
* Convert_String_to_Upper_Case end executable code *
***************************************************/

/******************************************************************************
* Module:                                                                     *
*    dtotal                                                                   *
*                                                                             *
* File:                                                                       *
*    generic.c                                                                *
*                                                                             *
* History:                                                                    *
*    Created    : --------- by -. -------. (unknown)                          *
*       Converted from GASP to C.                                             *
*    Delivered  : --------- by -. -------. (unknown)                          *
*                                                                             *
*    Updated    : 02-DEC-93 by J. Doggett,                                    *
*       Adopted into software used to facilitate the accessing of data from   *
*          the CD-ROM set of the Digitized Sky Survey.                        *
*    Redelivered: 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
*    Updated    : 28-NOV-94 by J. Doggett,                                    *
*       Corrected declaration of local variable d to double from errant int.  *
*    Redelivered: 30-NOV-94 by J. Doggett,                                    *
*       GetImage Version 1.1                                                  *
*                                                                             *
* Description:                                                                *
*    Computes the sum of the elements of a double array.                      *
*                                                                             *
* Return Value:                                                               *
*    double: The resulting sum                                                *
*                                                                             *
* Arguments:                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       a                                                                     *
*          double array, Reference                                            *
*             Array to sum.                                                   *
*       n                                                                     *
*          int, Value                                                         *
*             Number of element in the array.                                 *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
******************************************************************************/
extern double
dtotal(a,n)
double a[];
int n;
{
int    i;
double d;

      d = 0.0;
      for (i=0; i<n; i++) d += a[i];
      return(d);

} /* dtotal */

/******************************************************************************
* Module:                                                                     *
*    Handle_Fatal_Error                                                       *
*                                                                             *
* File:                                                                       *
*    generic.c                                                                *
*                                                                             *
* Description:                                                                *
*    Prints an error message includng the name of the function where the      *
*    error occurred and exits the program.                                    *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Function                                                              *
*          STRING, Reference                                                  *
*             Name of the function in which the error condition occurred.     *
*       Message                                                               *
*          STRING, Reference                                                  *
*             Error message to be printed.                                    *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 01-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/**********************************
* Handle_Fatal_Error declarations *
**********************************/
void   Handle_Fatal_Error( Function, Message )
STRING Function;
STRING Message ;
{
/*******************************************
* Handle_Fatal_Error begin executable code *
*******************************************/
   printf( "\nFATAL ERROR" );

   if ( strlen( Function ) != 0 )
      if ( strlen( Message ) != 0 )
         printf( " in %s:\n\n\t\"%s\"\n", Function, Message );
      else
         printf( " in %s.\n", Function );
   else
      if ( strlen( Message ) != 0 )
         printf( ":\n\n\t\"%s\"\n", Message );
      else
         printf( ".\n" );

   printf( "\nProgram aborted.\n\n" );

/****************************
* Handle_Fatal_Error page 2 *
****************************/

   exit( EXIT_FAILURE );
}
/*****************************************
* Handle_Fatal_Error end executable code *
*****************************************/

/******************************************************************************
* Module:                                                                     *
*    remquotes                                                                *
*                                                                             *
* File:                                                                       *
*    generic.c                                                                *
*                                                                             *
* History:                                                                    *
*    Created    : --------- by -. -------. (unknown)                          *
*       Converted from GASP to C.                                             *
*    Delivered  : --------- by -. -------. (unknown)                          *
*                                                                             *
*    Updated    : 02-DEC-93 by J. Doggett,                                    *
*       Adopted into software used to facilitate the accessing of data from   *
*          the CD-ROM set of the Digitized Sky Survey.                        *
*    Redelivered: 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
* Description:                                                                *
*    Strips ' from a string.                                                  *
*                                                                             *
* Return Value:                                                               *
*    char *: Pointer to resulting string.                                     *
*                                                                             *
* Arguments:                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       string                                                                *
*          char, Reference                                                    *
*             String to strip.                                                *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
******************************************************************************/
extern char
*remquotes(string)
char string[];
{
char *i1, *i2;

    i1 = strchr(string,'\'');
    if (i1 == NULL) return(string);
    i2 = strrchr(string,'\'');
    if (i2 == i1) return(string);
    i1 += 1;
    *i2 = '\0';
    return(i1);

} /* remquotes */

/******************************************************************************
* Module:                                                                     *
*    strtrim                                                                  *
*                                                                             *
* File:                                                                       *
*    generic.c                                                                *
*                                                                             *
* History:                                                                    *
*    Created    : --------- by -. -------. (unknown)                          *
*       Converted from GASP to C.                                             *
*    Delivered  : --------- by -. -------. (unknown)                          *
*                                                                             *
*    Updated    : 02-DEC-93 by J. Doggett,                                    *
*       Adopted into software used to facilitate the accessing of data from   *
*          the CD-ROM set of the Digitized Sky Survey.                        *
*    Updated    : 06-JAN-94 by J. Doggett,                                    *
*       BLANK macro changed from BLANKS.                                      *
*    Redelivered: 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
* Description:                                                                *
*    Removes leading blanks from a  string.                                   *
*                                                                             *
* Return Value:                                                               *
*    char *: Pointer to resulting string.                                     *
*                                                                             *
* Arguments:                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       string                                                                *
*          char, Reference                                                    *
*             String to strip.                                                *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
******************************************************************************/
extern char
*strtrim(string)
char string[];
{
int c;

      c = strspn(string, BLANK);
      return(&(string[c]));

} /* strtrim */

/******************************************************************************
* Module:                                                                     *
*    Translate_Environment_Variable                                           *
*                                                                             *
* File:                                                                       *
*    generic.c                                                                *
*                                                                             *
* Description:                                                                *
*    Returns the value of an environment variable using getenv unless returns *
*    NULL. Then an empty string is returned.                                  *
*                                                                             *
* Return Value:                                                               *
*    STRING: The translated environment variable.                             *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Variable                                                              *
*          STRING, Reference                                                  *
*             Environment variable to translate.                              *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 20-AUG-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/**********************************************
* Translate_Environment_Variable declarations *
**********************************************/
STRING Translate_Environment_Variable( Variable )
STRING Variable;
{
   STRING Translated_Value; /* Value of Variable translated with getenv */
   STRING Return_Value    ; /* Pointer to new memory with variable      */

/*******************************************************
* Translate_Environment_Variable begin executable code *
*******************************************************/
   Translated_Value = getenv( Variable );
   if ( Translated_Value == ( int )NULL )
      Translated_Value = "";

   Return_Value = ( STRING )Allocate_Memory( strlen( Translated_Value ) + 1 );
   strcpy( Return_Value, Translated_Value );

   return Return_Value;
}
/*****************************************************
* Translate_Environment_Variable end executable code *
*****************************************************/

/******************************************************************************
* Module:                                                                     *
*    Trim_Trailing_White_Space                                                *
*                                                                             *
* File:                                                                       *
*    generic.c                                                                *
*                                                                             *
* Description:                                                                *
*    Trim trailing white space from string. Trailing white space is white     *
*    preceding the terminating NULL. All trailing white space is converted to *
*    NULL's.                                                                  *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       none                                                                  *
*    Input/Output Arguments:                                                  *
*       String                                                                *
*          STRING, Reference                                                  *
*             String to trim.                                                 *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 25-AUG-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/*****************************************
* Trim_Trailing_White_Space declarations *
*****************************************/
void   Trim_Trailing_White_Space( String )
STRING String;
{
   int Nth_Char; /* Character in string we're investigating */

/**************************************************
* Trim_Trailing_White_Space begin executable code *
**************************************************/
   Nth_Char = strlen( String ) - 1;

   while( ( Nth_Char >= 0 ) && ( isspace( String[ Nth_Char ] ) ) )
      String[ Nth_Char-- ] = ( char )NULL;

   return;
}
/************************************************
* Trim_Trailing_White_Space end executable code *
************************************************/
